@@ -5,28 +5,29 @@ function(x)
   old.par  <- par(c('pty','mar','xpd','bg','xaxs','las','col.axis','fg'))
   on.exit(par(old.par))
 
-  # layout page
-  if(x@windows == 1) {
-    layout(matrix(1))
-  } else {
-    layout(matrix(1:x@windows,x@windows,1,byrow=TRUE),
-                  1,c(3,rep(1,x@windows-2),1.65),
-           respect=FALSE)
-    if(x@windows > 1) par(mar=c(0,4,3,3))
-  }
+  LAYOUT <- ifelse(is.null(x@layout),FALSE,TRUE)
 
+  par.list <- list(list(mar=c(  0,3.5,2,3)),
+                   list(mar=c(  0,3.5,0,3)),
+                   list(mar=c(3.5,3.5,0,3)))
+ 
+  # layout page
+  if(LAYOUT) {
+    if(!inherits(x@layout,'chart.layout')) {
+      cl <- chart.layout(x@windows)
+    } else cl <- x@layout
+    
+    layout(cl$mat, cl$width, cl$height, respect=FALSE)
+  }
+  if(x@windows > 1) do.call('par',par.list[[1]]) 
   #layout(matrix(c(2,3,4,5,1,1,1,1),nc=4,byrow=TRUE),c(1,1,1,1),c(1,2),FALSE)
   #layout(matrix(c(1,2,1,3,1,4,1,5),nc=2,byrow=TRUE),c(5,1),c(1),FALSE)
 
-
-
   x.range <- 1:(x@xrange[2]*x@spacing)
   y.range <- seq(x@yrange[1],x@yrange[2],length.out=length(x.range))
  
   # get current values of series to be charted
   xx <- x@xdata
-
-#  if(!is.xts(xx)) xx <- as.xts(xx)
  
   xx <- xx[x@xsubset]
 
@@ -37,8 +38,8 @@ function(x)
     Closes <- as.numeric(Cl(xx))
   } else {
     # if not OHLC, assume univariate series
-    Lows <- min(xx[,1])
-    Highs <- max(xx[,1])
+    Lows <- min(xx[,1],na.rm=TRUE)
+    Highs <- max(xx[,1],na.rm=TRUE)
     Closes <- as.numeric(xx[,1])
   }
 
@@ -48,6 +49,9 @@ function(x)
   # create scale of main plot window
   plot(x.range,y.range,type='n',axes=FALSE,ann=FALSE)
 
+  coords <- par('usr')
+  rect(coords[1],coords[3],coords[2],coords[4],col=x@colors$area)
+
   # check for any underlay TA indicators that need to be drawn here:
   main.key <- list() # main.key stores text to be added after all drawing by text()
   if (length(x@passed.args$TA) > 0) {
@@ -65,8 +69,12 @@ function(x)
   # a vector of x positions
   x.pos <- 1+x@spacing*(1:x@length-1)
 
+
   if(x@type=='line') {
-    lines(x.pos,Closes,col='blue',type=x@line.type)
+    lines(x.pos,Closes,col=x@colors$up.col,type=x@line.type)
+    main.key <- c(list(list(legend=
+                       paste('Last',last(Closes)),
+                       text.col=x@colors$up.col)),main.key)
   } else {
     # create a vector of colors
     if(x@multi.col) {
@@ -94,20 +102,6 @@ function(x)
     }
     if(x@type %in% c('candlesticks','matchsticks')) {
       # draw HL lines
-
-# in progress thinking...
-# if subsetting is allowed, keep x.pos based on length of output
-#  _but_ subset the y-values with the index corresponding to the
-# desired sub-range
-#
-# most likely found with something like:
-#  sindex <- which(index(x) %in% index(x)[subset])
-#
-#  the default subset should be set to :: (the whole thing...)
-#
-#  so the following would read:
-#     segments(x.pos,Lows[sindex],x.pos,Highs[sindex],col=bar.border)
-
       segments(x.pos,Lows,x.pos,Highs,col=bar.border)
 
       # draw OC candles
@@ -127,11 +121,21 @@ function(x)
         segments(x.pos-x@spacing/6,Closes,x.pos,Closes,col=bar.col)
       } else segments(x.pos-x@spacing/6,Opens,x.pos,Opens,col=bar.col)
     }    
+    main.key <- c(list(list(legend=
+                       paste('Last',last(Closes)),
+                       text.col=last(bar.col))),main.key)
   }
 
   axis(2)
   box(col=x@colors$fg.col)
-  do.call('title',list(x@name,col.main=x@colors$main.col))
+
+  old.adj <- par('adj')
+  par('adj'=0)
+  do.call('title',list(x@name, col.main=x@colors$fg.col))
+  par('adj'=1)
+  do.call('title',list(paste('[',start(xx),'/',end(xx),']', sep='')
+                      ,col.main=x@colors$main.col))
+  par('adj'=old.adj)
 
   # TA calculation and drawing loops
   if(x@windows > 1 | length(x@passed.args$TA) > 0) {
@@ -143,18 +147,15 @@ function(x)
         # call draws TA and returns the text to add to the chart
         overlay.text <- do.call(x@passed.args$TA[[j]]@name,list(x@passed.args$TA[[j]]))
         main.key <- c(main.key,overlay.text)
-        #main.key <- lapply(list(main.key,overlay.text),unlist)
-        #main.key[[length(main.key)+1]] <- overlay.text
       }
 
       if(i == 1) {
         # add indicator key to main chart
         if(length(main.key) > 0) {
           for(indicator in 1:length(main.key)) {
-            text(0,max(Closes,na.rm=TRUE),
-                 paste(paste(rep("\n\n\n",indicator),collapse=''),main.key[[indicator]][['text']],sep=''),
-                 pos=4, col=main.key[[indicator]][['col']]
-                )
+            legend("topleft",
+                   legend=c(rep('',indicator-1), paste(main.key[[indicator]][["legend"]],collapse="")),
+                   text.col=rev(main.key[[indicator]][["text.col"]])[1], bty='n', y.inter=0.95)
           }
         }
       }
@@ -162,8 +163,10 @@ function(x)
       if(x@windows >= i+1) {
         # if there are more windows to draw...draw the next one
         next.new.TA <- which(sapply(x@passed.args$TA,function(x) x@new))[i]
-        par(mar=c(0,4,0,3))
-        if(x@windows == i+1) par(mar=c(4,4,0,3))
+        do.call('par',par.list[[2]]) #par(mar=c(0,4,0,3))
+        if(x@windows == i+1) do.call('par',par.list[[3]]) #par(mar=c(4,4,0,3))
+#  coords <- par('usr')
+#  rect(coords[1],coords[3],coords[2],coords[4],col=x@colors$area)
         do.call(x@passed.args$TA[[next.new.TA]]@name,list(x@passed.args$TA[[next.new.TA]]))
       }
     }
@@ -171,16 +174,65 @@ function(x)
   }
 
   # draw the final x labels
-  #title(xlab=x@colors$time.scale,col.lab=x@colors$fg.col)
   if(x@minor.ticks)
     axis(1,at=x.pos,labels=FALSE,col=x@colors$minor.tick)
  
-  axis(1,at=x@bp*x@spacing+1,labels=x@x.labels,las=1,lwd=1,mgp=c(3,2,0),
+  axis(1,at=1+x@bp*x@spacing-x@spacing,labels=x@x.labels,las=1,lwd=1,mgp=c(3,2,0),
        col=x@colors$major.tick)
 
 
   # resave new chob object - just in case of any changes
   write.chob(x,dev.cur())
   # reset layout of page
-  layout(matrix(1))
+  if(LAYOUT) layout(matrix(1))
 }#}}}
+
+# chart.layout {{{
+`chart.layout` <-
+function(x) {
+  if(x==1) {
+    lyt <- 'layout(matrix(1))'
+    mat <- matrix(1)
+    wd  <- 1
+    ht  <- 1
+  } else {
+    lyt <- paste('layout(matrix(c(1,1:',x,'),',x+1,',1,byrow=TRUE),',
+                 '1,1,respect=FALSE)',sep='')
+                 #'1,c(3,rep(1,',x-2,'),1.60),respect=FALSE)',sep='')
+    mat <- matrix(1:x,x,1,byrow=TRUE)
+    wd  <- 1
+    ht  <- c(3,rep(1,x-2),1.60)
+  }
+  par.list <- list(list(mar=c(  0,3.5,2,3)),
+                   list(mar=c(  0,3.5,0,3)),
+                   list(mar=c(3.5,3.5,0,3)))
+  structure(list(text=lyt,mat=mat,width=wd,height=ht,par.list=par.list), class='chart.layout')
+}
+#}}}
+
+# experimental {{{
+#`doCharts` <- function(x, nc) {
+#  chartLayout(x,nc)
+#  for(i in 1:x) barChart(GS, subset='2008', layout=NULL)
+#}
+#
+#`chartLayout` <- function(x, nc) {
+# x <- (rep(c(1,1,2),x) + rep(seq(0,x*2-2,by=2),each=3),nc=nc,byrow=FALSE)
+# layout(x,1,1,respect=FALSE)
+#}
+#
+#`dozenCharts` <- function(x, nc) {
+#  getSymbols("GS")
+#  chartLayout(x,nc)
+#  TAs <- paste('addVo();addMACD();addRSI();addSMI();addROC();addDPO()',
+#               'addADX();addATR();addCMF();addCCI();addCMO();addWPR()',sep=';')
+#  TAs <- unlist(strsplit(TAs,';'))
+#  Overlays <- paste('addEMA();addBBands();addEnvelope()',
+#                     'addExpiry();addSAR();addSMA()',sep=';')
+#  Overlays <- rep(unlist(strsplit(Overlays,';')),2)
+#   
+#  for(i in 1:x) {
+#    TA <- paste(TAs[i],Overlays[i],sep=';')
+#    candleChart(GS, theme='white', subset='2008', type='b', layout=NULL, TA=TA)
+#  }
+#} #}}}
